@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_ITEM_H__
@@ -43,16 +43,22 @@ struct _GimpItemClass
 
   /*  signals  */
   void            (* removed)               (GimpItem            *item);
-  void            (* linked_changed)        (GimpItem            *item);
+  void            (* visibility_changed)    (GimpItem            *item);
   void            (* color_tag_changed)     (GimpItem            *item);
   void            (* lock_content_changed)  (GimpItem            *item);
   void            (* lock_position_changed) (GimpItem            *item);
+  void            (* lock_visibility_changed) (GimpItem            *item);
 
   /*  virtual functions  */
   void            (* unset_removed)      (GimpItem               *item);
   gboolean        (* is_attached)        (GimpItem               *item);
-  gboolean        (* is_content_locked)  (GimpItem               *item);
-  gboolean        (* is_position_locked) (GimpItem               *item);
+  gboolean        (* is_content_locked)  (GimpItem               *item,
+                                          GimpItem              **locked_item);
+  gboolean        (* is_position_locked) (GimpItem               *item,
+                                          GimpItem              **locked_item,
+                                          gboolean                checking_children);
+  gboolean        (* is_visibility_locked) (GimpItem               *item,
+                                            GimpItem              **locked_item);
   GimpItemTree  * (* get_tree)           (GimpItem               *item);
   gboolean        (* bounds)             (GimpItem               *item,
                                           gdouble                *x,
@@ -68,9 +74,17 @@ struct _GimpItemClass
                                           const gchar            *new_name,
                                           const gchar            *undo_desc,
                                           GError                **error);
+  void            (* start_move)         (GimpItem               *item,
+                                          gboolean                push_undo);
+  void            (* end_move)           (GimpItem               *item,
+                                          gboolean                push_undo);
+  void            (* start_transform)    (GimpItem               *item,
+                                          gboolean                push_undo);
+  void            (* end_transform)      (GimpItem               *item,
+                                          gboolean                push_undo);
   void            (* translate)          (GimpItem               *item,
-                                          gint                    offset_x,
-                                          gint                    offset_y,
+                                          gdouble                 offset_x,
+                                          gdouble                 offset_y,
                                           gboolean                push_undo);
   void            (* scale)              (GimpItem               *item,
                                           gint                    new_width,
@@ -104,6 +118,8 @@ struct _GimpItemClass
                                           GimpInterpolationType   interpolation_type,
                                           GimpTransformResize     clip_result,
                                           GimpProgress           *progress);
+  GimpTransformResize (* get_clip)       (GimpItem               *item,
+                                          GimpTransformResize     clip_result);
   gboolean        (* fill)               (GimpItem               *item,
                                           GimpDrawable           *drawable,
                                           GimpFillOptions        *fill_options,
@@ -206,9 +222,19 @@ void            gimp_item_set_offset         (GimpItem           *item,
 gint            gimp_item_get_offset_x       (GimpItem           *item);
 gint            gimp_item_get_offset_y       (GimpItem           *item);
 
+void            gimp_item_start_move         (GimpItem           *item,
+                                              gboolean            push_undo);
+void            gimp_item_end_move           (GimpItem           *item,
+                                              gboolean            push_undo);
+
+void            gimp_item_start_transform    (GimpItem           *item,
+                                              gboolean            push_undo);
+void            gimp_item_end_transform      (GimpItem           *item,
+                                              gboolean            push_undo);
+
 void            gimp_item_translate          (GimpItem           *item,
-                                              gint                offset_x,
-                                              gint                offset_y,
+                                              gdouble             offset_x,
+                                              gdouble             offset_y,
                                               gboolean            push_undo);
 
 gboolean        gimp_item_check_scaling      (GimpItem           *item,
@@ -226,6 +252,16 @@ gboolean        gimp_item_scale_by_factors   (GimpItem           *item,
                                               gdouble             h_factor,
                                               GimpInterpolationType interpolation,
                                               GimpProgress       *progress);
+gboolean
+      gimp_item_scale_by_factors_with_origin (GimpItem           *item,
+                                              gdouble             w_factor,
+                                              gdouble             h_factor,
+                                              gint                origin_x,
+                                              gint                origin_y,
+                                              gint                new_origin_x,
+                                              gint                new_origin_y,
+                                              GimpInterpolationType interpolation,
+                                              GimpProgress       *progress);
 void            gimp_item_scale_by_origin    (GimpItem           *item,
                                               gint                new_width,
                                               gint                new_height,
@@ -259,15 +295,17 @@ void            gimp_item_transform          (GimpItem           *item,
                                               GimpInterpolationType interpolation_type,
                                               GimpTransformResize clip_result,
                                               GimpProgress       *progress);
+GimpTransformResize   gimp_item_get_clip     (GimpItem           *item,
+                                              GimpTransformResize clip_result);
 
 gboolean        gimp_item_fill               (GimpItem           *item,
-                                              GimpDrawable       *drawable,
+                                              GList              *drawables,
                                               GimpFillOptions    *fill_options,
                                               gboolean            push_undo,
                                               GimpProgress       *progress,
                                               GError            **error);
 gboolean        gimp_item_stroke             (GimpItem           *item,
-                                              GimpDrawable       *drawable,
+                                              GList              *drawables,
                                               GimpContext        *context,
                                               GimpStrokeOptions  *stroke_options,
                                               GimpPaintOptions   *paint_options,
@@ -287,8 +325,8 @@ void            gimp_item_add_offset_node    (GimpItem           *item,
 void            gimp_item_remove_offset_node (GimpItem           *item,
                                               GeglNode           *node);
 
-gint            gimp_item_get_ID             (GimpItem           *item);
-GimpItem      * gimp_item_get_by_ID          (Gimp               *gimp,
+gint            gimp_item_get_id             (GimpItem           *item);
+GimpItem      * gimp_item_get_by_id          (Gimp               *gimp,
                                               gint                id);
 
 GimpTattoo      gimp_item_get_tattoo         (GimpItem           *item);
@@ -317,38 +355,46 @@ void            gimp_item_parasite_detach    (GimpItem           *item,
                                               gboolean            push_undo);
 const GimpParasite * gimp_item_parasite_find (GimpItem           *item,
                                               const gchar        *name);
-gchar        ** gimp_item_parasite_list      (GimpItem           *item,
-                                              gint               *count);
+gchar        ** gimp_item_parasite_list      (GimpItem           *item);
 
-void            gimp_item_set_visible        (GimpItem           *item,
+gboolean        gimp_item_set_visible        (GimpItem           *item,
                                               gboolean            visible,
                                               gboolean            push_undo);
 gboolean        gimp_item_get_visible        (GimpItem           *item);
 gboolean        gimp_item_is_visible         (GimpItem           *item);
 
-void            gimp_item_set_linked         (GimpItem           *item,
-                                              gboolean            linked,
-                                              gboolean            push_undo);
-gboolean        gimp_item_get_linked         (GimpItem           *item);
+void        gimp_item_bind_visible_to_active (GimpItem           *item,
+                                              gboolean            bind);
 
 void            gimp_item_set_color_tag      (GimpItem           *item,
                                               GimpColorTag        color_tag,
                                               gboolean            push_undo);
 GimpColorTag    gimp_item_get_color_tag      (GimpItem           *item);
+GimpColorTag  gimp_item_get_merged_color_tag (GimpItem           *item);
 
 void            gimp_item_set_lock_content   (GimpItem           *item,
                                               gboolean            lock_content,
                                               gboolean            push_undo);
 gboolean        gimp_item_get_lock_content   (GimpItem           *item);
 gboolean        gimp_item_can_lock_content   (GimpItem           *item);
-gboolean        gimp_item_is_content_locked  (GimpItem           *item);
+gboolean        gimp_item_is_content_locked  (GimpItem           *item,
+                                              GimpItem          **locked_item);
 
 void            gimp_item_set_lock_position  (GimpItem          *item,
                                               gboolean           lock_position,
                                               gboolean           push_undo);
 gboolean        gimp_item_get_lock_position  (GimpItem          *item);
 gboolean        gimp_item_can_lock_position  (GimpItem          *item);
-gboolean        gimp_item_is_position_locked (GimpItem          *item);
+gboolean        gimp_item_is_position_locked (GimpItem          *item,
+                                              GimpItem         **locked_item);
+
+void            gimp_item_set_lock_visibility  (GimpItem        *item,
+                                                gboolean         lock_visibility,
+                                                gboolean         push_undo);
+gboolean        gimp_item_get_lock_visibility  (GimpItem        *item);
+gboolean        gimp_item_can_lock_visibility  (GimpItem        *item);
+gboolean        gimp_item_is_visibility_locked (GimpItem        *item,
+                                                GimpItem       **locked_item);
 
 gboolean        gimp_item_mask_bounds        (GimpItem           *item,
                                               gint               *x1,
