@@ -38,6 +38,8 @@
 #include "core/gimpimage-pick-item.h"
 #include "core/gimpitem.h"
 
+#include "menus/menus.h"
+
 #include "widgets/gimpaction.h"
 #include "widgets/gimpcontrollers.h"
 #include "widgets/gimpcontrollerkeyboard.h"
@@ -61,6 +63,7 @@
 
 #include "gimpcanvas.h"
 #include "gimpdisplay.h"
+#include "gimpdisplay-foreach.h"
 #include "gimpdisplayshell.h"
 #include "gimpdisplayshell-autoscroll.h"
 #include "gimpdisplayshell-cursor.h"
@@ -144,7 +147,7 @@ static void       gimp_display_shell_untransform_event_coords (GimpDisplayShell
                                                                GimpCoords        *image_coords,
                                                                gboolean          *update_software_cursor);
 
-static void       gimp_display_shell_activate_action          (GimpUIManager     *manager,
+static void       gimp_display_shell_activate_action          (Gimp              *gimp,
                                                                const gchar       *action_desc,
                                                                GVariant          *value);
 
@@ -171,6 +174,15 @@ gimp_display_shell_events (GtkWidget        *widget,
 
   gimp = gimp_display_get_gimp (shell->display);
 
+  /* When a display is being created, we may have some weird cases where
+   * we get never-ending focus events fighting for which image window
+   * should have focus, in an infinite loop. See #11957.
+   * Just ignore focus events in these cases, and in particular, don't
+   * set the display from such focus events.
+   */
+  if (! gimp_displays_accept_focus_events (gimp))
+    return TRUE;
+
   switch (event->type)
     {
     case GDK_KEY_PRESS:
@@ -273,8 +285,7 @@ gimp_display_shell_canvas_no_image_events (GtkWidget        *canvas,
         GdkEventButton  *bevent = (GdkEventButton *) event;
         if (bevent->button == 1)
           {
-            GimpImageWindow *window  = gimp_display_shell_get_window (shell);
-            GimpUIManager   *manager = gimp_image_window_get_ui_manager (window);
+            GimpUIManager *manager = menus_get_image_manager_singleton (shell->display->gimp);
 
             gimp_ui_manager_activate_action (manager, "file", "file-open");
           }
@@ -286,7 +297,8 @@ gimp_display_shell_canvas_no_image_events (GtkWidget        *canvas,
       if (gdk_event_triggers_context_menu (event))
         {
           gimp_ui_manager_ui_popup_at_pointer (shell->popup_manager,
-                                               "/dummy-menubar/image-popup",
+                                               "/image-menubar",
+                                               GTK_WIDGET (shell),
                                                (GdkEvent *) event,
                                                NULL, NULL);
           return TRUE;
@@ -398,7 +410,7 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
           return TRUE;
 
         /*  ignore enter notify while we have a grab  */
-        if (shell->grab_pointer)
+        if (shell->grab_seat)
           return TRUE;
 
         gimp_display_shell_proximity_in (shell);
@@ -419,7 +431,7 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
           return TRUE;
 
         /*  ignore leave notify while we have a grab  */
-        if (shell->grab_pointer)
+        if (shell->grab_seat)
           return TRUE;
 
         gimp_display_shell_proximity_out (shell);
@@ -459,7 +471,7 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
               g_warning ("%s: FOCUS_IN but canvas has no focus", G_STRFUNC);
 
             /*  ignore focus changes while we have a grab  */
-            if (shell->grab_pointer)
+            if (shell->grab_seat)
               return TRUE;
 
             /*   press modifier keys when the canvas gets the focus  */
@@ -472,7 +484,7 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
               g_warning ("%s: FOCUS_OUT but canvas has focus", G_STRFUNC);
 
             /*  ignore focus changes while we have a grab  */
-            if (shell->grab_pointer)
+            if (shell->grab_seat)
               return TRUE;
 
             /*  release modifier keys when the canvas loses the focus  */
@@ -490,7 +502,7 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
         /*  ignore new mouse events  */
         if (gimp->busy                                     ||
             shell->mod_action != GIMP_MODIFIER_ACTION_NONE ||
-            shell->grab_pointer                            ||
+            shell->grab_seat                               ||
             shell->button1_release_pending)
           return TRUE;
 
@@ -585,6 +597,7 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
               {
               case GIMP_MODIFIER_ACTION_MENU:
                 gimp_display_triggers_context_menu (event, shell, gimp, &image_coords, TRUE);
+                shell->mod_action = GIMP_MODIFIER_ACTION_NONE;
                 break;
               case GIMP_MODIFIER_ACTION_PANNING:
               case GIMP_MODIFIER_ACTION_ZOOMING:
@@ -690,7 +703,7 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
              *  a button press we intentionally ignored because we had
              *  a grab on another device at the time of the press
              */
-            if (! shell->grab_pointer || shell->mod_action != GIMP_MODIFIER_ACTION_NONE)
+            if (! shell->grab_seat || shell->mod_action != GIMP_MODIFIER_ACTION_NONE)
               return TRUE;
 
             if (active_tool &&
@@ -725,12 +738,6 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
           }
         else
           {
-            GimpImageWindow *window;
-            GimpUIManager   *manager;
-
-            window  = gimp_display_shell_get_window (shell);
-            manager = gimp_image_window_get_ui_manager (window);
-
             switch (shell->mod_action)
               {
               case GIMP_MODIFIER_ACTION_MENU:
@@ -752,7 +759,7 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
                 gimp_display_shell_stop_scrolling (shell, event);
                 break;
               case GIMP_MODIFIER_ACTION_ACTION:
-                gimp_display_shell_activate_action (manager, shell->mod_action_desc, NULL);
+                gimp_display_shell_activate_action (gimp, shell->mod_action_desc, NULL);
                 g_clear_pointer (&shell->mod_action_desc, g_free);
                 break;
               case GIMP_MODIFIER_ACTION_NONE:
@@ -1208,7 +1215,7 @@ gimp_display_shell_canvas_tool_events (GtkWidget        *canvas,
                     /* We need to ungrab the pointer in order to catch
                      * button release events.
                      */
-                    if (shell->grab_pointer)
+                    if (shell->grab_seat)
                       gimp_display_shell_pointer_ungrab (shell, event);
                   }
                 else
@@ -1623,7 +1630,8 @@ gimp_display_shell_check_device (GimpDisplayShell *shell,
           event->type != GDK_KEY_RELEASE &&
           event->type != GDK_FOCUS_CHANGE)
         {
-          if ((shell->grab_pointer && (shell->grab_pointer != grab_device)) ||
+          if (shell->grab_seat &&
+              (shell->grab_pointer && shell->grab_pointer != grab_device) &&
               (shell->grab_pointer_source && (shell->grab_pointer_source != device)))
             {
               GIMP_LOG (TOOL_EVENTS,
@@ -1806,7 +1814,7 @@ gimp_display_shell_stop_scrolling (GimpDisplayShell *shell,
   /* We may have ungrabbed the pointer when space was released while
    * mouse was down, to be able to catch a GDK_BUTTON_RELEASE event.
    */
-  if (shell->grab_pointer)
+  if (shell->grab_seat)
     gimp_display_shell_pointer_ungrab (shell, event);
 }
 
@@ -1890,10 +1898,7 @@ gimp_display_shell_handle_scrolling (GimpDisplayShell *shell,
           action = gimp_tool_control_get_action_pixel_size (active_tool->control);
           if (action)
             {
-              GimpImageWindow *window  = gimp_display_shell_get_window (shell);
-              GimpUIManager   *manager = gimp_image_window_get_ui_manager (window);
-
-              gimp_display_shell_activate_action (manager, action,
+              gimp_display_shell_activate_action (gimp, action,
                                                   g_variant_new_double ((gdouble) size * size_multiplier));
 
               if (size_update_pos)
@@ -1915,14 +1920,11 @@ gimp_display_shell_handle_scrolling (GimpDisplayShell *shell,
 
               if (action)
                 {
-                  GimpImageWindow *window  = gimp_display_shell_get_window (shell);
-                  GimpUIManager   *manager = gimp_image_window_get_ui_manager (window);
-
                   /* Special trick with these enum actions. If using any
                    * positive value, we get the GIMP_ACTION_SELECT_SET behavior
                    * which sets to the given value.
                    */
-                  gimp_display_shell_activate_action (manager, action,
+                  gimp_display_shell_activate_action (gimp, action,
                                                       g_variant_new_int32 (size));
                 }
             }
@@ -1946,14 +1948,11 @@ gimp_display_shell_handle_scrolling (GimpDisplayShell *shell,
 
           if (action)
             {
-              GimpImageWindow *window  = gimp_display_shell_get_window (shell);
-              GimpUIManager   *manager = gimp_image_window_get_ui_manager (window);
-
               /* Special trick with these enum actions. If using any
                * positive value, we get the GIMP_ACTION_SELECT_SET behavior
                * which sets to the given value.
                */
-              gimp_display_shell_activate_action (manager, action,
+              gimp_display_shell_activate_action (gimp, action,
                                                   g_variant_new_int32 (size));
             }
         }
@@ -2116,9 +2115,8 @@ static gboolean
 gimp_display_shell_tab_pressed (GimpDisplayShell  *shell,
                                 const GdkEventKey *kevent)
 {
-  GimpImageWindow *window  = gimp_display_shell_get_window (shell);
-  GimpUIManager   *manager = gimp_image_window_get_ui_manager (window);
-  GimpImage       *image   = gimp_display_get_image (shell->display);
+  GimpUIManager *manager = menus_get_image_manager_singleton (shell->display->gimp);
+  GimpImage     *image   = gimp_display_get_image (shell->display);
 
   if (kevent->state & GDK_CONTROL_MASK)
     {
@@ -2305,11 +2303,9 @@ gimp_display_shell_initialize_tool (GimpDisplayShell *shell,
                *  just calling gimp_context_tool_changed(). See
                *  GimpGeglProcedure and bug #776370.
                */
-              GimpImageWindow *window;
-              GimpUIManager   *manager;
+              GimpUIManager *manager;
 
-              window  = gimp_display_shell_get_window (shell);
-              manager = gimp_image_window_get_ui_manager (window);
+              manager = menus_get_image_manager_singleton (shell->display->gimp);
 
               gimp_filter_history_add (gimp, procedure);
               gimp_ui_manager_activate_action (manager, "filters",
@@ -2402,42 +2398,37 @@ gimp_display_shell_untransform_event_coords (GimpDisplayShell *shell,
 }
 
 static void
-gimp_display_shell_activate_action (GimpUIManager *manager,
-                                    const gchar   *action_desc,
-                                    GVariant      *value)
+gimp_display_shell_activate_action (Gimp        *gimp,
+                                    const gchar *action_name,
+                                    GVariant    *value)
 {
-  gchar *group_name;
-  gchar *action_name;
-
-  g_return_if_fail (action_desc != NULL);
-
-  group_name  = g_strdup (action_desc);
-  action_name = strchr (group_name, '/');
+  g_return_if_fail (action_name != NULL);
 
   if (action_name)
     {
-      GimpAction *action;
+      GAction *action;
 
-      *action_name++ = '\0';
+      action = g_action_map_lookup_action (G_ACTION_MAP (gimp->app), action_name);
 
-      action = gimp_ui_manager_find_action (manager, group_name, action_name);
-
-      if (GIMP_IS_ENUM_ACTION (action) &&
-          GIMP_ENUM_ACTION (action)->value_variable)
+      if (action == NULL)
         {
-          gimp_action_emit_activate (action, value);
+          g_printerr ("%s: ignoring unknown action '%s'.\n",
+                      G_STRFUNC, action_name);
+        }
+      else if (GIMP_IS_ENUM_ACTION (action) &&
+               GIMP_ENUM_ACTION (action)->value_variable)
+        {
+          gimp_action_emit_activate (GIMP_ACTION (action), value);
         }
       else if (GIMP_IS_DOUBLE_ACTION (action))
         {
-          gimp_action_emit_activate (action, value);
+          gimp_action_emit_activate (GIMP_ACTION (action), value);
         }
       else
         {
-          gimp_action_activate (action);
+          gimp_action_activate (GIMP_ACTION (action));
         }
     }
-
-  g_free (group_name);
 }
 
 /* Replace gdk_event_triggers_context_menu() as we don't want to trigger
@@ -2487,11 +2478,12 @@ gimp_display_triggers_context_menu (const GdkEvent   *event,
           if (! ui_manager)
             {
               ui_manager = shell->popup_manager;
-              ui_path    = "/dummy-menubar/image-popup";
+              ui_path    = "/image-menubar";
             }
 
-          gimp_ui_manager_ui_popup_at_pointer (ui_manager, ui_path, event,
-                                               NULL, NULL);
+          gimp_ui_manager_ui_popup_at_pointer (ui_manager, ui_path,
+                                               GTK_WIDGET (shell),
+                                               event, NULL, NULL);
           return TRUE;
         }
     }
