@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #if !defined (__GIMP_H_INSIDE__) && !defined (GIMP_COMPILATION)
@@ -30,19 +30,21 @@ G_BEGIN_DECLS
 /* For information look into the C source or the html documentation */
 
 
-gint32   gimp_selection_float (gint32 image_ID,
-                               gint32 drawable_ID,
-                               gint   offx,
-                               gint   offy);
+#include <libgimp/gimpchannel.h>
 
-#ifndef GIMP_DISABLE_DEPRECATED
 
-gboolean gimp_selection_clear (gint32 image_ID);
+#define GIMP_TYPE_SELECTION (gimp_selection_get_type ())
+G_DECLARE_FINAL_TYPE (GimpSelection, gimp_selection, GIMP, SELECTION, GimpChannel)
 
-#endif /* GIMP_DISABLE_DEPRECATED */
+
+GimpSelection * gimp_selection_get_by_id (gint32        selection_id);
+
+GimpLayer     * gimp_selection_float     (GimpImage    *image,
+                                          GimpDrawable **drawables,
+                                          gint          offx,
+                                          gint          offy);
 
 
 G_END_DECLS
 
 #endif /* __GIMP_SELECTION_H__ */
-
