@@ -26,9 +26,20 @@ GimpImage * load_image  (GFile        *file,
                          gboolean      merged_image_only,
                          gboolean     *resolution_loaded,
                          gboolean     *profile_loaded,
+                         PSDSupport   *unsupported_features,
                          GError      **error);
 
-void        load_dialog (void);
+GimpImage * load_image_metadata
+                        (GFile        *file,
+                         gint          data_length,
+                         GimpImage    *image,
+                         gboolean      for_layers,
+                         gboolean      is_cmyk,
+                         PSDSupport   *unsupported_features,
+                         GError      **error);
+
+void        load_dialog (const gchar  *title,
+                         PSDSupport   *unsupported_features);
 
 
 #endif /* __PSD_LOAD_H__ */
